<?php

namespace EmailKing;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Updater {

	private $file;
	private $plugin;
	private $basename;
	private $slug;
	private $active;
	private $info;
	private $plan;

	use Internal;

	public static function init( $file ) {
		return new self( $file );
	}

	public function __construct( $file ) {
		$this->file = $file;

		add_action( 'admin_init', array( $this, 'set_plugin_properties' ) );
		add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'modify_transient' ), 10, 1 );
		add_filter( 'plugins_api', array( $this, 'plugin_popup' ), 10, 3 );
		add_filter( 'upgrader_post_install', array( $this, 'after_install' ), 10, 3 );
	}

	public function set_plugin_properties() {
		$this->plugin   = \get_plugin_data( $this->file );
		$this->basename = \plugin_basename( $this->file );
		$this->active   = \is_plugin_active( $this->basename );
		$this->slug     = \dirname( $this->basename );
		$this->plan     = 'free';
	}

	public function get_info() {
		$scope    = sprintf( 'email/plugin', $this->plan );
		$key      = sprintf( 'update-%s', $this->plan );
		$validity = 604800; // 7 Days.

		$this->info = $this->main( 'data' )->get( $scope, $key, null, $validity );

		if ( ! $this->info ) {
			$op = substr_count( EMAIL_KING_WP__API_UPDATE_INFO, '?' ) > 0 ? '&' : '?';

			$res = \wp_remote_get(
				EMAIL_KING_WP__API_UPDATE_INFO . $op . 'plan=' . $this->plan,
				array(
					'timeout' => 5,
				)
			);

			if ( ! \is_wp_error( $res ) ) {
				$body = \wp_remote_retrieve_body( $res );
				$body = \json_decode( $body, true );

				if ( is_array( $body ) && isset( $body['status'] ) && 200 === $body['status'] ) {
					$body['payload']['check']['slug'] = $this->slug;
					$body['payload']['popup']['slug'] = $this->slug;

					$this->info = $body['payload'];

					$this->main( 'data' )->set( $scope, $key, $this->info );
				}
			}
		}

		return $this->info;
	}

	public function modify_transient( $transient ) {

		if ( property_exists( $transient, 'checked' ) ) {

			if ( $transient->checked ) {

				$info = $this->get_info();

				if ( $info && is_array( $info ) && isset( $info['check'], $info['check']['new_version'] ) ) {
					$version  = $info['check']['new_version'];
					$outdated = version_compare( $this->plugin['Version'], $version, '<' );

					if ( $outdated ) {
						$transient->response[ $this->basename ] = (object) $info['check'];
					}
				}
			}
		}

		return $transient;
	}

	public function plugin_popup( $result, $action, $args ) {
		if ( is_object( $args ) && property_exists( $args, 'slug' ) && $args->slug === $this->slug ) {
			$info = $this->get_info();

			if ( $info ) {
				return (object) $info['popup'];
			}
		}

		return $result;
	}

	public function after_install( $response, $hook_extra, $result ) {
		global $wp_filesystem;

		$install_dir = plugin_dir_path( $this->file );

		$wp_filesystem->move( $result['destination'], $install_dir );

		$result['destination'] = $install_dir;

		if ( $this->active ) {
			activate_plugin( $this->basename );
		}

		return $result;
	}
}
